@@ -1,29 +1,27 @@
 """
 Copyright 2013 Steven Diamond
 
-This file is part of CVXPY.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-CVXPY is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-CVXPY is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with CVXPY.  If not, see <http://www.gnu.org/licenses/>.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 """
 
 from __future__ import print_function
-from cvxpy import *
+import cvxpy as cvx
 import cvxpy.interface as intf
-import numpy as np
 from cvxpy.tests.base_test import BaseTest
-import cvxopt
-import numbers
+from cvxpy.reductions.solvers.conic_solvers import ecos_conif
+import numpy as np
+import unittest
+
 
 class TestExamples(BaseTest):
     """ Unit tests using example problems. """
@@ -31,78 +29,79 @@ class TestExamples(BaseTest):
     # Find the largest Euclidean ball in the polyhedron.
     def test_chebyshev_center(self):
         # The goal is to find the largest Euclidean ball (i.e. its center and
-        # radius) that lies in a polyhedron described by linear inequalites in this
+        # radius) that lies in a polyhedron described by linear inequalities in this
         # fashion: P = {x : a_i'*x <= b_i, i=1,...,m} where x is in R^2
 
         # Generate the input data
-        a1 = np.matrix("2; 1")
-        a2 = np.matrix(" 2; -1")
-        a3 = np.matrix("-1;  2")
-        a4 = np.matrix("-1; -2")
+        a1 = np.array([2, 1])
+        a2 = np.array([2, -1])
+        a3 = np.array([-1, 2])
+        a4 = np.array([-1, -2])
         b = np.ones(4)
 
         # Create and solve the model
-        r = Variable(name='r')
-        x_c = Variable(2,name='x_c')
-        obj = Maximize(r)
-        constraints = [ #TODO have atoms compute values for constants.
-            a1.T*x_c + np.linalg.norm(a1)*r <= b[0],
-            a2.T*x_c + np.linalg.norm(a2)*r <= b[1],
-            a3.T*x_c + np.linalg.norm(a3)*r <= b[2],
-            a4.T*x_c + np.linalg.norm(a4)*r <= b[3],
+        r = cvx.Variable(name='r')
+        x_c = cvx.Variable(2, name='x_c')
+        obj = cvx.Maximize(r)
+        constraints = [  # TODO have atoms compute values for constants.
+            a1.T @ x_c + np.linalg.norm(a1)*r <= b[0],
+            a2.T @ x_c + np.linalg.norm(a2)*r <= b[1],
+            a3.T @ x_c + np.linalg.norm(a3)*r <= b[2],
+            a4.T @ x_c + np.linalg.norm(a4)*r <= b[3],
         ]
 
-        p = Problem(obj, constraints)
+        p = cvx.Problem(obj, constraints)
         result = p.solve()
         self.assertAlmostEqual(result, 0.447214)
         self.assertAlmostEqual(r.value, result)
-        self.assertItemsAlmostEqual(x_c.value, [0,0])
+        self.assertItemsAlmostEqual(x_c.value, [0, 0])
 
     # Test issue with numpy scalars.
     def test_numpy_scalars(self):
         n = 6
         eps = 1e-6
-        cvxopt.setseed(10)
-        P0 = cvxopt.normal(n, n)
-        eye = cvxopt.spmatrix(1.0, range(n), range(n))
-        P0 = P0.T * P0 + eps * eye
+        np.random.seed(10)
+        P0 = np.random.randn(n, n)
+        eye = np.eye(n)
+        P0 = P0.T.dot(P0) + eps * eye
 
         print(P0)
 
-        P1 = cvxopt.normal(n, n)
-        P1 = P1.T*P1
-        P2 = cvxopt.normal(n, n)
-        P2 = P2.T*P2
-        P3 = cvxopt.normal(n, n)
-        P3 = P3.T*P3
+        P1 = np.random.randn(n, n)
+        P1 = P1.T.dot(P1)
+        P2 = np.random.randn(n, n)
+        P2 = P2.T.dot(P2)
+        P3 = np.random.randn(n, n)
+        P3 = P3.T.dot(P3)
 
-        q0 = cvxopt.normal(n, 1)
-        q1 = cvxopt.normal(n, 1)
-        q2 = cvxopt.normal(n, 1)
-        q3 = cvxopt.normal(n, 1)
+        q0 = np.random.randn(n, 1)
+        q1 = np.random.randn(n, 1)
+        q2 = np.random.randn(n, 1)
+        q3 = np.random.randn(n, 1)
 
-        r0 = cvxopt.normal(1, 1)
-        r1 = cvxopt.normal(1, 1)
-        r2 = cvxopt.normal(1, 1)
-        r3 = cvxopt.normal(1, 1)
+        r0 = np.random.randn(1, 1)
+        r1 = np.random.randn(1, 1)
+        r2 = np.random.randn(1, 1)
+        r3 = np.random.randn(1, 1)
 
-        slack = Variable()
+        slack = cvx.Variable()
         # Form the problem
-        x = Variable(n)
-        objective = Minimize( 0.5*quad_form(x,P0) + q0.T*x + r0 + slack)
-        constraints = [0.5*quad_form(x,P1) + q1.T*x + r1 <= slack,
-                       0.5*quad_form(x,P2) + q2.T*x + r2 <= slack,
-                       0.5*quad_form(x,P3) + q3.T*x + r3 <= slack,
-        ]
+        x = cvx.Variable(n)
+        objective = cvx.Minimize(0.5*cvx.quad_form(x, P0) + q0.T @ x + r0 + slack)
+        constraints = [0.5*cvx.quad_form(x, P1) + q1.T @ x + r1 <= slack,
+                       0.5*cvx.quad_form(x, P2) + q2.T @ x + r2 <= slack,
+                       0.5*cvx.quad_form(x, P3) + q3.T @ x + r3 <= slack,
+                       ]
 
         # We now find the primal result and compare it to the dual result
         # to check if strong duality holds i.e. the duality gap is effectively zero
-        p = Problem(objective, constraints)
-        primal_result = p.solve()
+        p = cvx.Problem(objective, constraints)
+        p.solve()
 
-        # Note that since our data is random, we may need to run this program multiple times to get a feasible primal
+        # Note that since our data is random,
+        # we may need to run this program multiple times to get a feasible primal
         # When feasible, we can print out the following values
-        print(x.value) # solution
+        print(x.value)  # solution
         lam1 = constraints[0].dual_value
         lam2 = constraints[1].dual_value
         lam3 = constraints[2].dual_value
@@ -113,27 +112,26 @@ class TestExamples(BaseTest):
         r_lam = r0 + lam1*r1 + lam2*r2 + lam3*r3
         dual_result = -0.5*q_lam.T.dot(P_lam).dot(q_lam) + r_lam
         print(dual_result.shape)
-        self.assertEquals(intf.size(dual_result), (1,1))
+        self.assertEqual(intf.shape(dual_result), (1, 1))
 
     # Tests examples from the README.
     def test_readme_examples(self):
-        import cvxopt
         import numpy
-
-        # Problem data.
+        numpy.random.seed(1)
+        # cvx.Problem data.
         m = 30
         n = 20
-        A = cvxopt.normal(m,n)
-        b = cvxopt.normal(m)
+        A = numpy.random.randn(m, n)
+        b = numpy.random.randn(m)
 
         # Construct the problem.
-        x = Variable(n)
-        objective = Minimize(sum_entries(square(A*x - b)))
+        x = cvx.Variable(n)
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
-        p = Problem(objective, constraints)
+        p = cvx.Problem(objective, constraints)
 
         # The optimal objective is returned by p.solve().
-        result = p.solve()
+        p.solve()
         # The optimal value for x is stored in x.value.
         print(x.value)
         # The optimal Lagrange multiplier for a constraint
@@ -143,94 +141,92 @@ class TestExamples(BaseTest):
         ####################################################
 
         # Scalar variable.
-        a = Variable()
+        a = cvx.Variable()
 
         # Column vector variable of length 5.
-        x = Variable(5)
+        x = cvx.Variable(5)
 
         # Matrix variable with 4 rows and 7 columns.
-        A = Variable(4, 7)
+        A = cvx.Variable((4, 7))
 
         ####################################################
 
         # Positive scalar parameter.
-        m = Parameter(sign="positive")
+        m = cvx.Parameter(nonneg=True)
 
         # Column vector parameter with unknown sign (by default).
-        c = Parameter(5)
+        cvx.Parameter(5)
 
         # Matrix parameter with negative entries.
-        G = Parameter(4, 7, sign="negative")
+        G = cvx.Parameter((4, 7), nonpos=True)
 
         # Assigns a constant value to G.
         G.value = -numpy.ones((4, 7))
 
         # Raises an error for assigning a value with invalid sign.
         with self.assertRaises(Exception) as cm:
-            G.value = numpy.ones((4,7))
-        self.assertEqual(str(cm.exception), "Invalid sign for Parameter value.")
+            G.value = numpy.ones((4, 7))
+        self.assertEqual(str(cm.exception), "Parameter value must be nonpositive.")
 
         ####################################################
-        a = Variable()
-        x = Variable(5)
+        a = cvx.Variable()
+        x = cvx.Variable(5)
 
         # expr is an Expression object after each assignment.
         expr = 2*x
         expr = expr - a
-        expr = sum_entries(expr) + norm(x, 2)
+        expr = cvx.sum(expr) + cvx.norm(x, 2)
 
         ####################################################
 
         import numpy as np
-        import cvxopt
-        from multiprocessing import Pool
 
-        # Problem data.
+        # cvx.Problem data.
         n = 10
         m = 5
-        A = cvxopt.normal(n,m)
-        b = cvxopt.normal(n)
-        gamma = Parameter(sign="positive")
+        A = np.random.randn(n, m)
+        b = np.random.randn(n)
+        gamma = cvx.Parameter(nonneg=True)
 
         # Construct the problem.
-        x = Variable(m)
-        objective = Minimize(sum_entries(square(A*x - b)) + gamma*norm(x, 1))
-        p = Problem(objective)
+        x = cvx.Variable(m)
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b) + gamma*cvx.norm(x, 1))
+        p = cvx.Problem(objective)
 
         # Assign a value to gamma and find the optimal x.
         def get_x(gamma_value):
             gamma.value = gamma_value
-            result = p.solve()
+            p.solve()
             return x.value
 
         gammas = np.logspace(-1, 2, num=2)
         # Serial computation.
-        x_values = [get_x(value) for value in gammas]
+        [get_x(value) for value in gammas]
 
         ####################################################
         n = 10
 
-        mu = cvxopt.normal(1, n)
-        sigma = cvxopt.normal(n,n)
-        sigma = sigma.T*sigma
-        gamma = Parameter(sign="positive")
+        mu = np.random.randn(1, n)
+        sigma = np.random.randn(n, n)
+        sigma = sigma.T.dot(sigma)
+        gamma = cvx.Parameter(nonneg=True)
         gamma.value = 1
-        x = Variable(n)
+        x = cvx.Variable(n)
 
         # Constants:
         # mu is the vector of expected returns.
         # sigma is the covariance matrix.
-        # gamma is a Parameter that trades off risk and return.
+        # gamma is a cvx.Parameter that trades off risk and return.
 
-        # Variables:
+        # cvx.Variables:
         # x is a vector of stock holdings as fractions of total assets.
 
-        expected_return = mu*x
-        risk = quad_form(x, sigma)
+        expected_return = mu @ x
+        risk = cvx.quad_form(x, sigma)
 
-        objective = Maximize(expected_return - gamma*risk)
-        p = Problem(objective, [sum_entries(x) == 1])
-        result = p.solve()
+        objective = cvx.Maximize(expected_return - gamma*risk)
+        p = cvx.Problem(objective, [cvx.sum(x) == 1])
+        p.solve()
 
         # The optimal expected return.
         print(expected_return.value)
@@ -245,102 +241,104 @@ class TestExamples(BaseTest):
         n = 10
         data = []
         for i in range(N):
-            data += [(1, cvxopt.normal(n, mean=1.0, std=2.0))]
+            data += [(1, np.random.normal(loc=1.0, scale=2.0, size=n))]
         for i in range(M):
-            data += [(-1, cvxopt.normal(n, mean=-1.0, std=2.0))]
+            data += [(-1, np.random.normal(loc=-1.0, scale=2.0, size=n))]
 
         # Construct problem.
-        gamma = Parameter(sign="positive")
+        gamma = cvx.Parameter(nonneg=True)
         gamma.value = 0.1
         # 'a' is a variable constrained to have at most 6 non-zero entries.
-        a = Variable(n)#mi.SparseVar(n, nonzeros=6)
-        b = Variable()
+        a = cvx.Variable(n)  # mi.SparseVar(n, nonzeros=6)
+        b = cvx.Variable()
 
-        slack = [pos(1 - label*(sample.T*a - b)) for (label, sample) in data]
-        objective = Minimize(norm(a, 2) + gamma*sum(slack))
-        p = Problem(objective)
-        # Extensions can attach new solve methods to the CVXPY Problem class.
-        #p.solve(method="admm")
+        slack = [cvx.pos(1 - label*(sample.T @ a - b)) for (label, sample) in data]
+        objective = cvx.Minimize(cvx.norm(a, 2) + gamma*sum(slack))
+        p = cvx.Problem(objective)
+        # Extensions can attach new solve methods to the CVXPY cvx.Problem class.
+        # p.solve(method="admm")
         p.solve()
 
         # Count misclassifications.
         errors = 0
         for label, sample in data:
-            if label*(sample.T*a - b).value < 0:
+            if label*(sample.T @ a - b).value < 0:
                 errors += 1
 
         print("%s misclassifications" % errors)
         print(a.value)
         print(b.value)
 
-    def test_advanded(self):
+    def test_advanced1(self):
         """Code from the advanced tutorial.
         """
         # Solving a problem with different solvers.
-        x = Variable(2)
-        obj = Minimize(x[0] + norm(x, 1))
+        x = cvx.Variable(2)
+        obj = cvx.Minimize(x[0] + cvx.norm(x, 1))
         constraints = [x >= 2]
-        prob = Problem(obj, constraints)
+        prob = cvx.Problem(obj, constraints)
 
         # Solve with ECOS.
-        prob.solve(solver=ECOS)
+        prob.solve(solver=cvx.ECOS)
         print("optimal value with ECOS:", prob.value)
         self.assertAlmostEqual(prob.value, 6)
 
-        # Solve with ECOS_BB.
-        prob.solve(solver=ECOS_BB)
-        print("optimal value with ECOS_BB:", prob.value)
-        self.assertAlmostEqual(prob.value, 6)
-
         # Solve with CVXOPT.
-        prob.solve(solver=CVXOPT)
-        print("optimal value with CVXOPT:", prob.value)
-        self.assertAlmostEqual(prob.value, 6)
+        if cvx.CVXOPT in cvx.installed_solvers():
+            prob.solve(solver=cvx.CVXOPT)
+            print("optimal value with CVXOPT:", prob.value)
+            self.assertAlmostEqual(prob.value, 6)
 
         # Solve with SCS.
-        prob.solve(solver=SCS)
+        prob.solve(solver=cvx.SCS)
         print("optimal value with SCS:", prob.value)
-        self.assertAlmostEqual(prob.value, 6, places=3)
+        self.assertAlmostEqual(prob.value, 6, places=2)
 
-        if GLPK in installed_solvers():
+        if cvx.CPLEX in cvx.installed_solvers():
+            # Solve with CPLEX.
+            prob.solve(solver=cvx.CPLEX)
+            print("optimal value with CPLEX:", prob.value)
+            self.assertAlmostEqual(prob.value, 6)
+
+        if cvx.GLPK in cvx.installed_solvers():
             # Solve with GLPK.
-            prob.solve(solver=GLPK)
+            prob.solve(solver=cvx.GLPK)
             print("optimal value with GLPK:", prob.value)
             self.assertAlmostEqual(prob.value, 6)
 
             # Solve with GLPK_MI.
-            prob.solve(solver=GLPK_MI)
+            prob.solve(solver=cvx.GLPK_MI)
             print("optimal value with GLPK_MI:", prob.value)
             self.assertAlmostEqual(prob.value, 6)
 
-        if GUROBI_LIN in installed_solvers():
+        if cvx.GUROBI in cvx.installed_solvers():
             # Solve with Gurobi.
-            prob.solve(solver=GUROBI_LIN)
-            print("optimal value with GUROBI_LIN:", prob.value)
+            prob.solve(solver=cvx.GUROBI)
+            print("optimal value with GUROBI:", prob.value)
             self.assertAlmostEqual(prob.value, 6)
 
-        print(installed_solvers())
+        print(cvx.installed_solvers())
 
     def test_log_det(self):
         # Generate data
-        x = np.matrix("0.55  0.0;"
-                      "0.25  0.35;"
-                      "-0.2   0.2;"
-                      "-0.25 -0.1;"
-                      "-0.0  -0.3;"
-                      "0.4  -0.2").T
+        x = np.array([[0.55, 0.0],
+                      [0.25, 0.35],
+                      [-0.2, 0.2],
+                      [-0.25, -0.1],
+                      [-0.0, -0.3],
+                      [0.4, -0.2]]).T
         (n, m) = x.shape
 
         # Create and solve the model
-        A = Variable(n, n);
-        b = Variable(n);
-        obj = Maximize( log_det(A) )
+        A = cvx.Variable((n, n))
+        b = cvx.Variable(n)
+        obj = cvx.Maximize(cvx.log_det(A))
         constraints = []
         for i in range(m):
-            constraints.append( norm(A*x[:, i] + b) <= 1 )
-        p = Problem(obj, constraints)
+            constraints.append(cvx.norm(A @ x[:, i] + b) <= 1)
+        p = cvx.Problem(obj, constraints)
         result = p.solve()
-        self.assertAlmostEqual(result, 1.9746, places=4)
+        self.assertAlmostEqual(result, 1.9746, places=2)
 
     def test_portfolio_problem(self):
         """Test portfolio problem that caused dcp_attr errors.
@@ -348,10 +346,8 @@ class TestExamples(BaseTest):
         import numpy as np
         import scipy.sparse as sp
         np.random.seed(5)
-        n = 100#10000
-        m = 10#100
-        pbar = (np.ones((n, 1)) * .03 +
-                np.matrix(np.append(np.random.rand(n - 1, 1), 0)).T * .12)
+        n = 100  # 10000
+        m = 10  # 100
 
         F = sp.rand(m, n, density=0.01)
         F.data = np.ones(len(F.data))
@@ -360,35 +356,33 @@ class TestExamples(BaseTest):
         Z = np.random.randn(m, 1)
         Z = Z.dot(Z.T)
 
-        x = Variable(n)
-        y = x.__rmul__(F)
-        mu = 1
-        ret = pbar.T * x
+        x = cvx.Variable(n)
+        y = F @ x
         # DCP attr causes error because not all the curvature
         # matrices are reduced to constants when an atom
         # is scalar.
-        risk = square(norm(x.__rmul__(D))) + square(Z*y)
+        cvx.square(cvx.norm(D @ x)) + cvx.square(Z @ y)
 
     def test_intro(self):
         """Test examples from cvxpy.org introduction.
         """
         import numpy
 
-        # Problem data.
+        # cvx.Problem data.
         m = 30
         n = 20
         numpy.random.seed(1)
         A = numpy.random.randn(m, n)
-        b = numpy.random.randn(m, 1)
+        b = numpy.random.randn(m)
 
         # Construct the problem.
-        x = Variable(n)
-        objective = Minimize(sum_squares(A*x - b))
+        x = cvx.Variable(n)
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
-        prob = Problem(objective, constraints)
+        prob = cvx.Problem(objective, constraints)
 
         # The optimal objective is returned by p.solve().
-        result = prob.solve()
+        prob.solve()
         # The optimal value for x is stored in x.value.
         print(x.value)
         # The optimal Lagrange multiplier for a constraint
@@ -397,28 +391,6 @@ class TestExamples(BaseTest):
 
         ########################################
 
-        # Create two scalar variables.
-        x = Variable()
-        y = Variable()
-
-        # Create two constraints.
-        constraints = [x + y == 1,
-                       x - y >= 1]
-
-        # Form objective.
-        obj = Minimize(square(x - y))
-
-        # Form and solve problem.
-        prob = Problem(obj, constraints)
-        prob.solve()  # Returns the optimal value.
-        print("status:", prob.status)
-        print("optimal value", prob.value)
-        print("optimal var", x.value, y.value)
-
-        ########################################
-
-        import cvxpy as cvx
-
         # Create two scalar variables.
         x = cvx.Variable()
         y = cvx.Variable()
@@ -437,7 +409,27 @@ class TestExamples(BaseTest):
         print("optimal value", prob.value)
         print("optimal var", x.value, y.value)
 
-        self.assertEqual(prob.status, OPTIMAL)
+        ########################################
+
+        # Create two scalar variables.
+        x = cvx.Variable()
+        y = cvx.Variable()
+
+        # Create two constraints.
+        constraints = [x + y == 1,
+                       x - y >= 1]
+
+        # Form objective.
+        obj = cvx.Minimize(cvx.square(x - y))
+
+        # Form and solve problem.
+        prob = cvx.Problem(obj, constraints)
+        prob.solve()  # Returns the optimal value.
+        print("status:", prob.status)
+        print("optimal value", prob.value)
+        print("optimal var", x.value, y.value)
+
+        self.assertEqual(prob.status, cvx.OPTIMAL)
         self.assertAlmostEqual(prob.value, 1.0)
         self.assertAlmostEqual(x.value, 1.0)
         self.assertAlmostEqual(y.value, 0)
@@ -445,111 +437,113 @@ class TestExamples(BaseTest):
         ########################################
 
         # Replace the objective.
-        prob.objective = Maximize(x + y)
+        prob = cvx.Problem(cvx.Maximize(x + y), prob.constraints)
         print("optimal value", prob.solve())
 
-        self.assertAlmostEqual(prob.value, 1.0)
+        self.assertAlmostEqual(prob.value, 1.0, places=3)
 
         # Replace the constraint (x + y == 1).
-        prob.constraints[0] = (x + y <= 3)
+        constraints = prob.constraints
+        constraints[0] = (x + y <= 3)
+        prob = cvx.Problem(prob.objective, constraints)
         print("optimal value", prob.solve())
 
-        self.assertAlmostEqual(prob.value, 3.0)
+        self.assertAlmostEqual(prob.value, 3.0, places=2)
 
         ########################################
 
-        x = Variable()
+        x = cvx.Variable()
 
         # An infeasible problem.
-        prob = Problem(Minimize(x), [x >= 1, x <= 0])
+        prob = cvx.Problem(cvx.Minimize(x), [x >= 1, x <= 0])
         prob.solve()
         print("status:", prob.status)
         print("optimal value", prob.value)
 
-        self.assertEquals(prob.status, INFEASIBLE)
+        self.assertEqual(prob.status, cvx.INFEASIBLE)
         self.assertAlmostEqual(prob.value, np.inf)
 
         # An unbounded problem.
-        prob = Problem(Minimize(x))
+        prob = cvx.Problem(cvx.Minimize(x))
         prob.solve()
         print("status:", prob.status)
         print("optimal value", prob.value)
 
-        self.assertEquals(prob.status, UNBOUNDED)
+        self.assertEqual(prob.status, cvx.UNBOUNDED)
         self.assertAlmostEqual(prob.value, -np.inf)
 
         ########################################
 
         # A scalar variable.
-        a = Variable()
+        cvx.Variable()
 
         # Column vector variable of length 5.
-        x = Variable(5)
+        x = cvx.Variable(5)
 
         # Matrix variable with 4 rows and 7 columns.
-        A = Variable(4, 7)
+        A = cvx.Variable((4, 7))
 
         ########################################
         import numpy
 
-        # Problem data.
+        # cvx.Problem data.
         m = 10
         n = 5
         numpy.random.seed(1)
         A = numpy.random.randn(m, n)
-        b = numpy.random.randn(m, 1)
+        b = numpy.random.randn(m)
 
         # Construct the problem.
-        x = Variable(n)
-        objective = Minimize(sum_entries(square(A*x - b)))
+        x = cvx.Variable(n)
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
-        prob = Problem(objective, constraints)
+        prob = cvx.Problem(objective, constraints)
 
         print("Optimal value", prob.solve())
         print("Optimal var")
-        print(x.value) # A numpy matrix.
+        print(x.value)  # A numpy matrix.
 
         self.assertAlmostEqual(prob.value, 4.14133859146)
 
         ########################################
         # Positive scalar parameter.
-        m = Parameter(sign="positive")
+        m = cvx.Parameter(nonneg=True)
 
         # Column vector parameter with unknown sign (by default).
-        c = Parameter(5)
+        cvx.Parameter(5)
 
         # Matrix parameter with negative entries.
-        G = Parameter(4, 7, sign="negative")
+        G = cvx.Parameter((4, 7), nonpos=True)
 
         # Assigns a constant value to G.
         G.value = -numpy.ones((4, 7))
         ########################################
 
         # Create parameter, then assign value.
-        rho = Parameter(sign="positive")
+        rho = cvx.Parameter(nonneg=True)
         rho.value = 2
 
         # Initialize parameter with a value.
-        rho = Parameter(sign="positive", value=2)
+        rho = cvx.Parameter(nonneg=True, value=2)
 
         ########################################
 
         import numpy
 
-        # Problem data.
+        # cvx.Problem data.
         n = 15
         m = 10
         numpy.random.seed(1)
         A = numpy.random.randn(n, m)
-        b = numpy.random.randn(n, 1)
+        b = numpy.random.randn(n)
         # gamma must be positive due to DCP rules.
-        gamma = Parameter(sign="positive")
+        gamma = cvx.Parameter(nonneg=True)
 
         # Construct the problem.
-        x = Variable(m)
-        error = sum_squares(A*x - b)
-        obj = Minimize(error + gamma*norm(x, 1))
-        prob = Problem(obj)
+        x = cvx.Variable(m)
+        error = cvx.sum_squares(A @ x - b)
+        obj = cvx.Minimize(error + gamma*cvx.norm(x, 1))
+        prob = cvx.Problem(obj)
 
         # Construct a trade-off curve of ||Ax-b||^2 vs. ||x||_1
         sq_penalty = []
@@ -562,19 +556,19 @@ class TestExamples(BaseTest):
             # Use expr.value to get the numerical value of
             # an expression in the problem.
             sq_penalty.append(error.value)
-            l1_penalty.append(norm(x, 1).value)
+            l1_penalty.append(cvx.norm(x, 1).value)
             x_values.append(x.value)
 
         ########################################
         import numpy
 
-        X = Variable(5, 4)
+        X = cvx.Variable((5, 4))
         A = numpy.ones((3, 5))
 
         # Use expr.size to get the dimensions.
         print("dimensions of X:", X.size)
-        print("dimensions of sum_entries(X):", sum_entries(X).size)
-        print("dimensions of A*X:", (A*X).size)
+        print("dimensions of sum(X):", cvx.sum(X).size)
+        print("dimensions of A @ X:", (A @ X).size)
 
         # ValueError raised for invalid dimensions.
         try:
@@ -602,70 +596,53 @@ class TestExamples(BaseTest):
                     Known[i, j] = 1
         Ucorr = Known*Uorig
         # Recover the original image using total variation in-painting.
-        U = Variable(rows, cols)
-        obj = Minimize(tv(U))
-        constraints = [mul_elemwise(Known, U) == mul_elemwise(Known, Ucorr)]
-        prob = Problem(obj, constraints)
-        prob.solve(solver=SCS)
+        U = cvx.Variable((rows, cols))
+        obj = cvx.Minimize(cvx.tv(U))
+        constraints = [cvx.multiply(Known, U) == cvx.multiply(Known, Ucorr)]
+        prob = cvx.Problem(obj, constraints)
+        prob.solve(solver=cvx.SCS)
 
-    def test_advanced(self):
+    def test_advanced2(self):
         """Test code from the advanced section of the tutorial.
         """
-        x = Variable()
-        prob = Problem(Minimize(square(x)), [x == 2])
+        x = cvx.Variable()
+        prob = cvx.Problem(cvx.Minimize(cvx.square(x)), [x == 2])
         # Get ECOS arguments.
-        data = prob.get_problem_data(ECOS)
-
-        # Get ECOS_BB arguments.
-        data = prob.get_problem_data(ECOS_BB)
+        data, chain, inverse = prob.get_problem_data(cvx.ECOS)
 
         # Get CVXOPT arguments.
-        data = prob.get_problem_data(CVXOPT)
+        if cvx.CVXOPT in cvx.installed_solvers():
+            data, chain, inverse = prob.get_problem_data(cvx.CVXOPT)
 
         # Get SCS arguments.
-        data = prob.get_problem_data(SCS)
+        data, chain, inverse = prob.get_problem_data(cvx.SCS)
 
         import ecos
         # Get ECOS arguments.
-        data = prob.get_problem_data(ECOS)
+        data, chain, inverse = prob.get_problem_data(cvx.ECOS)
         # Call ECOS solver.
-        solver_output = ecos.solve(data["c"], data["G"], data["h"],
-                                   data["dims"], data["A"], data["b"])
+        solution = ecos.solve(data["c"], data["G"], data["h"],
+                              ecos_conif.dims_to_solver_dict(data["dims"]),
+                              data["A"], data["b"])
         # Unpack raw solver output.
-        prob.unpack_results(ECOS, solver_output)
+        prob.unpack_results(solution, chain, inverse)
 
-    # # Risk return tradeoff curve
-    # def test_risk_return_tradeoff(self):
-    #     from math import sqrt
-    #     from cvxopt import matrix
-    #     from cvxopt.blas import dot
-    #     from cvxopt.solvers import qp, options
-    #     import scipy
+    def test_log_sum_exp(self):
+        """Test log_sum_exp function that failed in Github issue.
+        """
+        import numpy as np
+        np.random.seed(1)
+        m = 5
+        n = 2
+        X = np.ones((m, n))
+        w = cvx.Variable(n)
 
-    #     n = 4
-    #     S = matrix( [[ 4e-2,  6e-3, -4e-3,   0.0 ],
-    #                  [ 6e-3,  1e-2,  0.0,    0.0 ],
-    #                  [-4e-3,  0.0,   2.5e-3, 0.0 ],
-    #                  [ 0.0,   0.0,   0.0,    0.0 ]] )
-    #     pbar = matrix([.12, .10, .07, .03])
+        expr2 = [cvx.log_sum_exp(cvx.hstack([0, X[i, :] @ w])) for i in range(m)]
+        expr3 = sum(expr2)
+        obj = cvx.Minimize(expr3)
+        p = cvx.Problem(obj)
+        p.solve(solver=cvx.SCS, max_iters=1)
 
-    #     N = 100
-    #     # CVXPY
-    #     Sroot = numpy.asmatrix(scipy.linalg.sqrtm(S))
-    #     x = cp.Variable(n, name='x')
-    #     mu = cp.Parameter(name='mu')
-    #     mu.value = 1 # TODO cp.Parameter("positive")
-    #     objective = cp.Minimize(-pbar*x + mu*quad_over_lin(Sroot*x,1))
-    #     constraints = [sum_entries(x) == 1, x >= 0]
-    #     p = cp.Problem(objective, constraints)
 
-    #     mus = [ 10**(5.0*t/N-1.0) for t in range(N) ]
-    #     xs = []
-    #     for mu_val in mus:
-    #         mu.value = mu_val
-    #         p.solve()
-    #         xs.append(x.value)
-    #     returns = [ dot(pbar,x) for x in xs ]
-    #     risks = [ sqrt(dot(x, S*x)) for x in xs ]
-
-    #     # QP solver
\ No newline at end of file
+if __name__ == '__main__':
+    unittest.main()
